function cg2ddemo(m)
%
% This program creates the pde example in the CG section. The example
% has the form
%
% Lu = - grad^T (a grad u) = f, 0 < x,y < 1
% with homogeneous Dirichlet BC
%
% In terms of the program below, the equation is
% - a1 (u_xx + u_yy) - a_x u_x - a_y u_y = f
% - (a u_x)_x - (a u_y)_y = f
% 
% The action of the laplacian, partial wrt x, and partial wrt y
% are computed in the routines lapmf, dxmf, and dymf. Here mf
% stands for MATRIX FREE.
%
global a1 ax ay zzz;
h=1/(m+1);
h2=(m+1)*(m+1);
h1=(m+1);
tol=1/h2;
%
% set up the equation
%
% Here a(x,y) = sqrt(.1+x)
%
x=1:m;
x=x'*h;
za=cos(x);
%
% coefficients for linellcg2
%
zx=0:m+1;
zx=zx'*h;
zb=cos(zx)*h2*.5;
zzz=zb*ones(1,m+2);
at=za*ones(1,m);
%axt=.5./at;
axt=-sin(x)*ones(1,m);
ayt=zeros(m,m);
%
% Form the solution: sol(x,y)= 10 x y (1-x) (1-y) exp(x^4.5)
%
zsoly=x.*(1-x);
zsolx=zsoly.*exp((1-x).^4.5);
solt=10*zsolx*zsoly';
a1=at(:);
ax=axt(:);
ay=ayt(:);
sol=solt(:);
clear rhs; clear axt; clear at; clear ayt; clear solt;
%
% fix the right side so that the solution is known
%
%b = a1.*lapmf(sol) - ax.*dxmf(sol) - ay.*dymf(sol);
b=linellcg2(sol);
%
% Set the initial iterate to zero. Call the solver twice, both
% with and without preconditioning. The preconditioner is the
% FFT-based poisson solver fish2d. See the source for details.
%
u0=zeros(m*m,1);
tic
%[u1, err, totits] = pcgsol(u0, b, @linellcg2, [tol, 10*m], 'fish2d');
cgoptions=kl_optset('lmethod','cg','ltol',tol,'maxitl',10*m,'ptv','fish2d');
[u, err, totits] = kl(u0, b, @linellcg2, cgoptions);
toc
ss=0:totits;
tic
%[u, err0, totits0] = pcgsol(u0, b, @linellcg2, [tol, 10*m]);
cgoptions=kl_optset('lmethod','cg','ltol',tol,'maxitl',10*m);
%[u, err0, totits0] = pcgsol(u0, b, @linellcg2, [tol, 10*m]);
[u, err0, totits0] = kl(u0, b, @linellcg2, cgoptions);
toc
ss=0:totits;
%
% plotting commands used to make the figures in the text
%
ss0=0:totits0;
semilogy(ss,err/norm(b),'-',ss0,err0/norm(b),'--');
phi=[];
xlabel('Iterations')
ylabel('Relative Residual')
legend('pc', 'no pc');

function lu=linellcg2(u)
%
% Matrix-free computation of linear elliptic operator for CG demo
%
% Lu = -a1 (u_xx + u_yy) - ax u_x - ay u_y
%
global a1 ax ay zzz;
n2=length(u);
n=sqrt(n2);
vu=zeros(n,n);
vu(:)=u;
uu=zeros(n+2,n+2);
uu(2:n+1,2:n+1)=vu;
h=1/(n+1);
h2=(n+1)*(n+1);
x=0:n+1;
x=x'*h;
%za=sqrt(.1+x)*h2*.5;
%ax1=za*ones(1,n+2);
ax1=zzz;
ax3=ax1(2:n+1,2:n+1);
a3=ax3(:);
bx=ax1(1:n+1,2:n+1)+ax1(2:n+2,2:n+1);
by=ax1(2:n+1,1:n+1)+ax1(2:n+1,2:n+2);
ux=uu(2:n+2,2:n+1)-uu(1:n+1,2:n+1);
vx=ux.*bx;
lux=vx(2:n+1,1:n)-vx(1:n,1:n);
uy=uu(2:n+1,2:n+2)-uu(2:n+1,1:n+1);
vy=uy.*by;
luy=vy(1:n,2:n+1)-vy(1:n,1:n);
lu=-lux(:)-luy(:);
